<?php
require "vendor/autoload.php";

/**
 * OxaPay Payment Gateway
 *
 * @package blesta
 * @subpackage blesta.components.gateways.nonmerchant.oxapay
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/  Blesta
 */

class oxapay extends NonmerchantGateway {
	private static $version = "1.0.0";
	private static $authors = array(array('name' => "OxaPay.com", 'url' => "https://oxapay.com"));
	private static $description = "OxaPay is a leading crypto payment gateway that enables businesses to accept payments in various cryptocurrencies securely. With OxaPay, merchants can expand their customer base by offering a convenient and efficient payment method that leverages the benefits of blockchain technology.";
	private static $signup_url = "https://app.OxaPay.com/signup";
	private $meta;

	public function __construct() {
		Loader::loadComponents($this, array("Input"));
		Language::loadLang("oxapay", null, dirname(__FILE__) . DS . "language" . DS);
	}

	public function getName() {
		return Language::_("OxaPay.name", true);
	}

	public function getVersion() {
		return self::$version;
	}

	public function getDescription() {
		return self::$description;
	}

	public function getSignupUrl() {
		return self::$signup_url;
	}

	public function getAuthors() {
		return self::$authors;
	}

	public function getCurrencies() {
	    $url = 'https://api.oxapay.com/api/fiats';

        $response = file_get_contents($url);
        $data = json_decode($response, true);
        $symbols = [];
        foreach ($data['data'] as $currency) {
            $symbols[] = $currency['symbol'];
        }
    
        return $symbols;                                        
	}

	public function setCurrency($currency) {
		$this->currency = $currency;
	}

	public function getSettings(array $meta=null) {
		$this->view = $this->makeView("settings", "default", str_replace(ROOTWEBDIR, "", dirname(__FILE__) . DS));
		Loader::loadHelpers($this, array("Form", "Html"));
		$this->view->set("meta", $meta);
		return $this->view->fetch();
	}

	public function editSettings(array $meta) {
		$rules = array(
			'merchant_key' => array(
				'empty' => array(
					'rule' => 'isEmpty',
					'negate' => true,
					'message' => Language::_("OxaPay.!error.merchant_key.required", true)
				)
			),
		);


		$this->Input->setRules($rules);
		$this->Input->validates($meta);
		return $meta;
	}

	public function encryptableFields() {
		return ["merchant_key"];
	}

	public function setMeta(array $meta=null) {
		$this->meta = $meta;
	}


	public function buildProcess(array $contact_info, $amount, array $invoice_amounts=null, array $options=null) {
		$this->view = $this->makeView("process", "default", str_replace(ROOTWEBDIR, "", dirname(__FILE__) . DS));
		Loader::loadHelpers($this, array("Form", "Html"));
		$fields = array();
		$post_to = "";
		$order_id = str_replace("invoice", "blesta", preg_replace("/[^a-zA-Z0-9]+/", "_", strtolower("amount_" . $amount . "__" . $options['description'])));
		$pos_data = $this->serializeInvoices($invoice_amounts);
		$url_success = $options['return_url'];
		$query = parse_url($url_success, PHP_URL_QUERY);
		$url_success .= $query ? '&' : '?';
		$url_success .= 'order_id=' . $order_id . '&posData=' . $pos_data;
		$url_callback = Configure::get('Blesta.gw_callback_url') . Configure::get('Blesta.company_id') . "/OxaPay/?";
		$url_callback .= 'client_id=' . $contact_info['client_id'] . '&order_id=' . $order_id . '&posData=' . $pos_data;

		$gatewayUrl = 'https://api.oxapay.com/merchants/request';
		$params = [
			'merchant' => $this->meta['merchant_key'],
			'amount' => (string)$amount,
			'currency' => $this->currency,
			'order_id' => $order_id,
			'callbackUrl' => $url_callback,
			'returnUrl' => $url_success,
			'lifeTime' => (string)((int)$this->meta['lifetime']),
		];
		$options = array(
			'http' => array(
				'header' => 'Content-Type: application/json',
				'method'  => 'POST',
				'content' => json_encode($params),
			),
		);
		$context  = stream_context_create($options);
		$response = file_get_contents($gatewayUrl, false, $context);
		$result = json_decode($response);
		$invoice_link = $result->payLink;

		$fields['url'] = $invoice_link;
		$url_success .= '&trackId=' . $result->trackId;
		$this->view->set("post_to", $post_to);
		$this->view->set("fields", $fields);
		return $this->view->fetch();
	}
	private function convertToBlestaStatus($status) {
		$result = 'declined';
		switch($status) {
			case 'Paid':
				$result = 'approved';
				break;

			case 'Wating':
			case 'Pending':
			case 'Confirming':
				$result = 'pending';
				break;

			case 'Failed':
				$result = 'error';
				break;

			case 'Cancel':
				$result = 'declined';
				break;
		}
		return $result;
	}

	public function validate(array $get, array $post) {
	    $postData = file_get_contents('php://input');
        $data = json_decode($postData, true);

        if ($data['type'] === 'payment') {
            $apiSecretKey = $this->meta['merchant_key'];
        } else {
            http_response_code(400);
            echo 'Invalid data.type';
            exit;
        }
        $hmacHeader = $_SERVER['HTTP_HMAC'];
        $calculatedHmac = hash_hmac('sha512', $postData, $apiSecretKey);
        if ($calculatedHmac === $hmacHeader) {
    		$invoices = [];
    		if (isset($get['posData'])) {
    			$invoices = $this->deSerializeInvoices($get['posData']);
    		}
    		$status = $this->convertToBlestaStatus($data['status']);
    		$transaction = [
    			'client_id' => ($get['client_id'] ?? null),
    			'amount' => ($data['amount'] ?? '0.0'),
    			'currency' => ($data['currency'] ?? null),
    			'invoices' => ($invoices ?? null),
    			'status' => $status,
    			'transaction_id' => ($data['trackId'] ?? null),
    			'parent_transaction_id' => null
    		];
            http_response_code(200);
    		return $transaction;
         }
         else {
            http_response_code(400);
            echo 'Invalid HMAC signature';
        }

	}


	public function success(array $get, array $post) {
		$invoices = [];
		if (isset($get['posData'])) {
			$invoices = $this->deSerializeInvoices($get['posData']);
		}
 
        if ($this->meta['merchant_key'] == 'sandbox' ){
    		return [
    			'client_id' => ($get['client_id'] ?? null),
    			'invoices' => ($invoices ?? null),
    			'status' => 'approved',
    			'transaction_id' => ($get['trackId'] ?? null),
    			'parent_transaction_id' => null
	    	];

        }
        else {
            $url = 'https://api.oxapay.com/merchants/inquiry';
    
            $data = array(
                'merchant' => $this->meta['merchant_key'],
                'trackId' => $get['trackId']
            );
            
            $options = array(
                'http' => array(
                    'header' => 'Content-Type: application/json',
                    'method'  => 'POST',
                    'content' => json_encode($data),
                ),
            );
            
            $context  = stream_context_create($options);
            $response = file_get_contents($url, false, $context);
            $result = json_decode($response);
            
    		$status = $this->convertToBlestaStatus($result->status);
    
    		return [
    			'client_id' => ($get['client_id'] ?? null),
    			'amount' => ($result->amount ?? '0.0'),
    			'currency' => ($result->currency ?? null),
    			'invoices' => ($invoices ?? null),
    			'status' => $status,
    			'transaction_id' => ($get['trackId'] ?? null),
    			'parent_transaction_id' => null
    		];

        }

	}

	public function capture($reference_id, $transaction_id, $amount, array $invoice_amounts=null) {
		$this->Input->setErrors($this->getCommonError("unsupported"));
	}

	public function void($reference_id, $transaction_id, $notes=null) {
		$this->Input->setErrors($this->getCommonError("unsupported"));
	}

	public function refund($reference_id, $transaction_id, $amount, $notes=null) {
		$this->Input->setErrors($this->getCommonError("unsupported"));
	}

	private function serializeInvoices(array $invoices) {
		$str = '';
		foreach ($invoices as $i => $invoice) {
			$temp = ($i > 0 ? '_' : '') . $invoice['id'] . '*' . $invoice['amount'];
			if (strlen($temp) + strlen($str) >= 69) {
				break;
			}
			$str .= $temp;
		}
		return $str;
	}

	private function deSerializeInvoices($str) {
		$invoices = [];
		$temp = explode('_', $str);
		foreach ($temp as $pair) {
			$pairs = explode('*', $pair, 2);
			if (count($pairs) != 2) {
				continue;
			}
			$invoices[] = ['id' => $pairs[0], 'amount' => $pairs[1]];
		}
		return $invoices;
	}
}
?>
